@@ -1,6 +1,6 @@
 from extras.plugins import PluginConfig
 
-__version__ = "0.16.1"
+__version__ = "0.17.0"
 
 
 class DNSConfig(PluginConfig):
@@ -21,6 +21,12 @@ class DNSConfig(PluginConfig):
         "zone_soa_expire": 2592000,
         "zone_soa_minimum": 3600,
         "feature_ipam_integration": False,
+        "tolerate_underscores_in_hostnames": False,
+        "tolerate_leading_underscore_types": [
+            "TXT",
+            "SRV",
+        ],
+        "tolerate_non_rfc1035_types": [],
     }
     base_url = "netbox-dns"
 
