@@ -1,7 +1,7 @@
 import graphene
 from graphene_django.converter import convert_django_field
 
-from netbox_dns.fields import NetworkField
+from netbox_dns.fields import NetworkField, AddressField
 
 
 @convert_django_field.register(NetworkField)
@@ -9,6 +9,11 @@ def convert_field_to_string(field, registry=None):
     return graphene.String(description=field.help_text, required=not field.null)
 
 
+@convert_django_field.register(AddressField)
+def convert_field_to_string(field, registry=None):
+    return graphene.String(description=field.help_text, required=not field.null)
+
+
 from .schema import *
 
 from .view import *
